#!/usr/bin/tcl -f

########################################
#                                      #
#      Group Matrix Extractor          #
#  Depth-First Search (DFS) Algorithm  #
#                                      #
#    COPYRIGHT(C) Alexander Kozik      #
#                                      #
#        December 30 2001              #
#                                      #
########################################

proc Adj_List_Extraction {current_line sourcefile2} {

    global query_file_open
    global cutoff_value

    set matrix_to_process [open "$sourcefile2" "r"]

    while {[gets $matrix_to_process current_matrix_line] >= 0}  {
	set matrix_id_A [lindex [split $current_matrix_line] 0]
	set matrix_id_B [lindex [split $current_matrix_line] 1]
	set current_sim_value [lindex [split $current_matrix_line] 2]
	if {$current_line == $matrix_id_A && $current_sim_value >= $cutoff_value} {
	    puts -nonewline $query_file_open $matrix_id_B\|
	}
	if {$current_line == $matrix_id_B && $current_sim_value >= $cutoff_value} {
	    puts -nonewline $query_file_open $matrix_id_A\|
	}
	if {$current_line != $matrix_id_A && $current_line != $matrix_id_B} {
	continue
	}
    }
    close $matrix_to_process
}

proc Already_Done {current_node_ID} {

	global used_nodes_list

        set already_done [lsearch -exact $used_nodes_list $current_node_ID]
                if {$already_done < 0} {
		    set used_nodes_list [lappend used_nodes_list "$current_node_ID"]
                }
                if {$already_done >= 0} {
		    continue
                }
}

proc DFS_Procedure {first_id m sub_group_open} {

global query_file
global used_nodes_list 
global group_file_open
global group_members_counter
global round_2_counter
global retarded_counter

    set l 1
    set group_counter 1
    set current_query_file_open [open "$query_file" "r"]
	while {[gets $current_query_file_open current_adj_line] >= 0} {
	    set current_adj_list [split $current_adj_line "|"]
	    set perfect_match [lsearch -exact $current_adj_list $first_id]
	    if {$perfect_match >= 0} {
		set current_adj_line_length [expr [llength [split $current_adj_line "|"]] - 1]
		while {$current_adj_line_length >= 0} {
		    set current_node_ID [lindex [split $current_adj_line "|"] [expr $current_adj_line_length -1]]
		    set already_done [lsearch -exact $used_nodes_list $current_node_ID]
		    if {$already_done < 0 && $current_node_ID != ""} {
			puts $group_file_open "$current_node_ID\t$m"
			puts $sub_group_open "$current_node_ID"
			puts "$current_node_ID\t$m\t$round_2_counter"
			incr group_members_counter
			incr round_2_counter
			Already_Done $current_node_ID
			DFS_Procedure $current_node_ID $m $sub_group_open
			incr retarded_counter
		    }
		    incr current_adj_line_length -1
		}
	    }
	incr l
	}
	close $current_query_file_open
puts "Escape from DSF ... $retarded_counter  times"
}

proc Finish_Dummy_Groups {current_id current_group group_begin} {

    global group_full_info_open
    global query_file
    global output_file

    set query_file_open [open "$query_file" "r"]

    set group_stat "$output_file\.group_stat"
    set group_stat_open [open $group_stat "r"]

    while {[gets $group_stat_open current_group_info] >= 0} {
	set current_group_info_id [lindex [split $current_group_info] 0]
	set current_group_info_stat [lindex [split $current_group_info] 1]
	if {$current_group == $current_group_info_id} {
	    set graph_complexity $current_group_info_stat
	}
    }

    while {[gets $query_file_open current_adj_line] >= 0} {
	set first_id [lindex [split $current_adj_line "|"] 0]
	set vertex_degree [expr [llength [split $current_adj_line "|"]] -2]
	if {$current_id == $first_id} {
	    puts $group_full_info_open "$current_id\t$vertex_degree\t$graph_complexity\t$current_group\t$group_begin"
	}
    }
    close $query_file_open
    close $group_stat_open
}

#############
# MAIN BODY #
#############

################################
# standard input/output dialog #

puts -nonewline "Enter the LIST file name: "
flush stdout

set sourcefile1 [gets stdin]

while {[catch {set f_in1 [open "$sourcefile1"  "r"]}] == 1} {
        puts stderr "Error opening $sourcefile"
        puts -nonewline "Please re-enter the LIST file name: "
        flush stdout
        set sourcefile1 [gets stdin]
}

puts -nonewline "Enter the MATRIX file name: "
flush stdout

set sourcefile2 [gets stdin]

while {[catch {set f_in2 [open "$sourcefile2"  "r"]}] == 1} {
        puts stderr "Error opening $sourcefile2"
        puts -nonewline "Please re-enter the SEQUNCE file name: "
        flush stdout
        set sourcefile2 [gets stdin]
}

close $f_in2

puts -nonewline "Enter the CUTOFF identity value: "
flush stdout
set cutoff_value [gets stdin]

# end of input/output dialog #
##############################

global output_file
global query_file
global query_file_open 
global used_nodes_list 
global group_file_open
global group_stat_open
global group_members_counter
global round_2_counter

######################
#      ROUND 1       #
######################

set id_to_process $f_in1

set file_output_extension "adj_list"
set output_file [lindex [split $sourcefile1 "."] 0]
set query_file "$output_file.$file_output_extension"
file delete -force "$query_file"
set query_file_open [open "$query_file" "a"]

set group_output_extension "group_info"
set group_file "$output_file.$group_output_extension"
file delete -force "$group_file"
set group_file_open [open "$group_file" "a"]

set k 1
    while {[gets $id_to_process current_line] >= 0}  {
	puts -nonewline $query_file_open $current_line\|
	puts $k
	Adj_List_Extraction $current_line $sourcefile2
	puts -nonewline $query_file_open \n
	incr k
    }
puts "Almost Done... End of Round 1"
#####################
close $id_to_process
close $query_file_open
######################
#      ROUND 2       #
######################

############

global retarded_counter

set retarded_counter 1
set round_2_counter 1
set group_stat "$output_file\.group_stat"
set group_stat_open [open $group_stat "w"]

set id_to_process [open "$sourcefile1" "r"]
set m 0
    while {[gets $id_to_process current_line] >= 0}  {
        if { $m == 0 } {
            set used_nodes_list [lappend used_nodes_list ""]
        }
        set already_done [lsearch -exact $used_nodes_list $current_line]
                if {$already_done < 0} {
                set used_nodes_list [lappend used_nodes_list "$current_line"]
                    set used_nodes_list_length [llength [split $used_nodes_list]]
                    incr m
		    set sub_group "subgroup_$m"
		    set sub_group_open [open $sub_group "w"]
		    set group_members_counter 1
		    puts -nonewline $group_stat_open "$m\t"
                    puts $group_file_open "$current_line\t$m\t*****"
		    puts $sub_group_open "$current_line"
		    DFS_Procedure $current_line $m $sub_group_open
		    puts "$current_line\t$m\t$round_2_counter"
		    close $sub_group_open
		    puts $group_stat_open "$group_members_counter"
		    incr round_2_counter
                }
                if {$already_done >= 0} {
                    set used_nodes_list_length [llength [split $used_nodes_list]]
                }
    }
close $group_file_open
close $id_to_process
close $group_stat_open
puts "Almost Done... End of Round 2"
###################
#     Round 3     #
###################
global group_full_info_open
set group_full_info "$output_file\.group_degree_info"
set group_full_info_open [open "$group_full_info" "w"]
set group_file_open [open "$group_file" "r"]
set p 1
while {[gets $group_file_open current_line] >= 0}  {
    set current_id [lindex [split $current_line] 0]
    set current_group [lindex [split $current_line] 1]
    set group_begin [lindex [split $current_line] 2]
    Finish_Dummy_Groups $current_id $current_group $group_begin
    puts $p
    incr p
}
close $group_full_info_open
close $group_file_open
puts "Done... End of Round 3"
